#ifndef CSIMNEUT_HH
#define CSIMNEUT_HH

#include "TObject.h"
#include "TVector3.h"

#include <deque>
#include <string>

#include "TSimulationDataScintillator.hh"
#include "TSimulationParameterNeutronDetector.hh"

class TDirectory;
class TFile;
class TH1;

// physical constants
static const Double_t mneutron = 939.565;
static const Double_t vlight = 29.979;

class CSimNeut
{
public:
  CSimNeut();
  virtual ~CSimNeut();
  
  int book(std::string rootfile);
  int SetOutputFileName(std::string rootfile);

  virtual int StartAnalysis(Int_t numberOfNeutrons = 1, Int_t vetoMode = 1, Int_t numberOfEntries = -1);
  virtual int ReadParameters();
  virtual int PrintParameters();
  virtual int CreateHist();
  virtual int CreateAndAnalyzeExpData();

protected:
  //calculate simlation data to experiment data
  virtual void SimToExp(std::vector<TSimulationDataScintillator>* simulationDataScintillatorArray);

protected:
  std::deque<std::string> fROOTFileNameArray;
  std::string fOutputFileName;

  TDirectory* fTopDirectory;
  TFile* fFileSim;
  TFile* fFileExp;

  Long64_t fNumberOfEntries;
  Long64_t fEventNumber;
  Int_t fNumberOfNeutrons;
  Int_t fVetoMode;

  TH1* fHIDQL;
  TH1* fHIDQR;
  TH1* fHIDTL;
  TH1* fHIDTR;
  TH1* fHIDQAve;
  TH1* fHIDTAve;
  std::vector<TH1*> fHTQ;
  std::vector<TH1*> fHQT;
  std::vector<TH1*> fHXY;

  TH1* fhenergy;
  TH1* fhenergytheta;
  TH1* fhenergy1n;
  TH1* fhq1q2;
  TH1* fhdtq1;
  TH1* fhdtq2;
  TH1* fhdbetaq1;
  TH1* fhdbetaq2;
  TH1* fhivdbetaq1;
  TH1* fhivdbetaq2;
  TH1* fhbetaddbetaq1;
  TH1* fhbetaddbetaq2;
  TH1* fhivbetaddbetaq1;
  TH1* fhivbetaddbetaq2;

  // neutron detector parameters
  TSimulationParameterNeutronDetector fSimPara;
//  TVector3 fNeutPos, fNeutPosShift, fNeutLayerSize, fNeutLayerPos, fNeutLayerPosShift;
//  TVector3 fVetoPos, fVetoPosShift, fVetoLayerSize, fVetoLayerPos, fVetoLayerPosShift;
//  Double_t fNeutLongSide, fNeutStuckSizeZ;
//  Double_t fVetoLongSide, fVetoStuckSizeZ;
//  TVector3 fStuckSize, fStuckPos, fStuckPosShift;

  Int_t fTotalNeutNumber;
  Int_t fTotalVetoNumber;
  Int_t fTotalModuleNumber;  

  Double_t fNeutHardwareThreshold;
  Double_t fVetoHardwareThreshold;
  Double_t fNeutSoftwareThreshold;
  Double_t fVetoSoftwareThreshold;
  Double_t fTimeResolution;
  Double_t fNeutAttenuationLength;
  Double_t fVetoAttenuationLength;
  Double_t fPropagationVelocity;
  Int_t fMultiplicity;

  TString fAnalysisMethod;

  // beam parameters
  Double_t fStandardKineticEenrgy;
  Double_t fStandardBeta;

  // temporary container
  std::vector<Int_t> fID;
  std::vector<Double_t> fQL, fQR;
  std::vector<Double_t> fTL, fTR;
};

#endif
