
#include "CSimulationTools.hh"

#include <string>
#include <vector>
#include <cmath>

#include "TRandom.h"

Double_t CSimulationTools::MeVtoMeVee(Double_t Tin, Double_t Tout, std::string name)
{
  Double_t a1, a2, a3, a4;
  Double_t Tein, Teout, dTe;

  if(name == "e-" || name == "e+" || name == "mu-" || name == "mu+"){//leptons
    a1 = 1;
    a2 = 0;
    a3 = 0;
    a4 = 0;
  }else if(name == "proton" || name == "deuteron" || name == "triton"){//Z=1
    //NE-102(CECIL et al.) 
    a1 = 0.95;
    a2 = 8.0;
    a3 = 0.1;
    a4 = 0.9;
  }else if(name == "alpha" || name == "He3"){//Z=2
    //plastic(CECIL et al.)
    a1 = 0.41;
    a2 = 5.9;
    a3 = 0.065;
    a4 = 1.01;
  }else{//heavy ions
    //used in DEMONS
    a1 = 0.29;
    a2 = 37.492;
    a3 = 0.01;
    a4 = 0.874;
  }

  Tein = a1*Tin-a2*(1-exp(-a3*pow(Tin, a4)));
  Teout = a1*Tout-a2*(1-exp(-a3*pow(Tout, a4)));
  dTe = Tein - Teout;

  if(dTe < 0) dTe = 0;

  return dTe;
}

void CSimulationTools::threshold(std::vector<CNeutData>& data_v, double neutSoftwareThreshold, double vetoSoftwareThreshold)
{
  for(int j=0; j<(int)data_v.size(); ++j){
    if(data_v[j].id <= 120){
      if(data_v[j].qave < neutSoftwareThreshold){
	data_v.erase(data_v.begin()+j);
	--j;
	continue;
      }
    }else{
      if(data_v[j].qave < vetoSoftwareThreshold){
	data_v.erase(data_v.begin()+j);
	--j;
	continue;
      }
    }
  }
}

void CSimulationTools::acceptance(std::vector<CNeutData>& data_v, double theta)
{
  for(int j=0; j<(int)data_v.size(); ++j){
    CNeutData data = data_v[j];
    double posxrand = data.pos[0]+ gRandom->Uniform(-6,6);
    double posy = data.pos[1];
    double posz = data.pos[2];
    double data_theta = atan(sqrt(posxrand*posxrand+posy*posy)/posz);
    if(!(data_theta < theta)){
      data_v.erase(data_v.begin()+j);
      --j;
      continue;
    }
  }
}

bool CSimulationTools::veto(std::vector<CNeutData>& data_v, int mode)
{
  bool vetoflag = false;
  for(int j=0; j<(int)data_v.size(); ++j){
    int id = data_v[j].id;
    if(mode == 1){
      if(id > 120 && id <= 132){
	vetoflag = true;
	break;
      }
    }else if(mode == 2){
      if(id > 120 && id <= 144){
	vetoflag = true;
	break;
      }
    }
  }

  return vetoflag;
}

void CSimulationTools::killveto(std::vector<CNeutData>& data_v)
{
  for(int j=0; j<(int)data_v.size(); ++j){
    int id = data_v[j].id;
    if(id > 120){ // VETO
      data_v.erase(data_v.begin()+j);
      --j;
    }
  }
}

void CSimulationTools::hitmintime(std::vector<CNeutData>& data_v)
{
  if(data_v.size() != 0){
    Double_t tmin = data_v[0].tave;
    while((int)data_v.size() > 1){
      Double_t t = data_v[1].tave;

      if(tmin > t){
	tmin = t;
	data_v.erase(data_v.begin());
      }else{
	data_v.erase(data_v.begin()+1);
      }
    }
  }
}

void CSimulationTools::hitmintime2(std::vector<CNeutData>& data_v)
{
  if(data_v.size() != 0){
    std::vector<std::vector<CNeutData> > data_v_layer(2);
    for(int j=0; j<(int)data_v.size(); ++j){
      int id = data_v[j].id;
      if(id <= 60){ // 1st layer neut
	data_v_layer[0].push_back(data_v[j]);
      }else if(id > 60 && id <= 120){ // 2nd layer neut
	data_v_layer[1].push_back(data_v[j]);
      }
    }

    if(data_v_layer[0].size() != 0){
      Double_t tmin = data_v_layer[0][0].tave;
      while((int)data_v_layer[0].size() > 1){
	Double_t t = data_v_layer[0][1].tave;

	if(tmin > t){
	  tmin = t;
	  data_v_layer[0].erase(data_v_layer[0].begin());
	}else{
	  data_v_layer[0].erase(data_v_layer[0].begin()+1);
	}
      }
    }

    if(data_v_layer[1].size() != 0){
      Double_t tmin = data_v_layer[1][0].tave;
      while((int)data_v_layer[1].size() > 1){
	Double_t t = data_v_layer[1][1].tave;

	if(tmin > t){
	  tmin = t;
	  data_v_layer[1].erase(data_v_layer[1].begin());
	}else{
	  data_v_layer[1].erase(data_v_layer[1].begin()+1);
	}
      }
    }

    data_v.clear();
    if(data_v_layer[0].size() == 1){
      data_v.push_back(data_v_layer[0][0]);
    }
    if(data_v_layer[1].size() == 1){
      data_v.push_back(data_v_layer[1][0]);
    }
  }
}
