
#include "ScintillatorSD.hh"

#include "G4HCofThisEvent.hh"
#include "G4SDManager.hh"
#include "G4VPhysicalVolume.hh"
#include "G4Step.hh"
#include "G4Track.hh"
#include "G4VTouchable.hh"
#include "G4TouchableHistory.hh"
#include "G4VProcess.hh"
#include "G4ParticleDefinition.hh"
#include "G4PrimaryParticle.hh"

#include "DetectorConstruction.hh"

#include "TSimulationDataScintillator.hh"

#include "TFile.h"
#include "TTree.h"
#include "TBranch.h"

ScintillatorSD::ScintillatorSD(G4String name) : G4VSensitiveDetector(name)
{;}

ScintillatorSD::~ScintillatorSD()
{;}

void ScintillatorSD::Initialize(G4HCofThisEvent* /*HCTE*/)
{;}

G4bool ScintillatorSD::ProcessHits(G4Step* aStep, G4TouchableHistory* /*ROhist*/)
{
  G4StepPoint* preStepPoint = aStep->GetPreStepPoint();
  G4StepPoint* postStepPoint = aStep->GetPostStepPoint();
  const G4DynamicParticle* dynamicParticle = aStep->GetTrack()->GetDynamicParticle();
  const G4ParticleDefinition* particleDefinition = aStep->GetTrack()->GetParticleDefinition();
  const G4VProcess* postProcess = postStepPoint->GetProcessDefinedStep();

  Double_t hitTime = preStepPoint->GetGlobalTime();
  Double_t hitEnergyDeposit = aStep->GetTotalEnergyDeposit();
  Double_t hitKineticEnergy = preStepPoint->GetKineticEnergy();
  TVector3 hitPosition(preStepPoint->GetPosition().x(),
		       preStepPoint->GetPosition().y(),
		       preStepPoint->GetPosition().z());
  TVector3 hitMomentum(preStepPoint->GetMomentum().x(),
		       preStepPoint->GetMomentum().y(),
		       preStepPoint->GetMomentum().z());

  G4int primaryParticleID = 0;  // particle ID needs to simulate more than 1 particle injection, but not yet
  G4int pdgCode = dynamicParticle->GetPDGcode();
  TString particleName(particleDefinition->GetParticleName().data());
  TString processName;
  if(postProcess) processName = postProcess->GetProcessName().data();

  G4TouchableHistory* theTouchable = (G4TouchableHistory*)(preStepPoint->GetTouchable());
  std::vector<Int_t> moduleID;
  for(G4int i=0; i<theTouchable->GetHistoryDepth(); ++i){
    G4int depth = i;//0:self, 1:mother, 2:grandmother, ....
    moduleID.push_back(theTouchable->GetCopyNumber(depth));
  }
  G4VPhysicalVolume* physicalVolume = theTouchable->GetVolume(0); // 0: depth
  const G4String& moduleName = physicalVolume->GetName();

  //  if(hitEnergyDeposit>0.){
    TSimulationDataScintillator simulationDataScintillator(moduleName, moduleID, 
							   primaryParticleID, pdgCode, particleName, processName,
							   1./cm*hitPosition, 1./MeV*hitMomentum,
							   hitKineticEnergy/MeV, hitTime/ns, hitEnergyDeposit/MeV);

    gSimulationDataScintillatorArray->push_back(simulationDataScintillator);

#ifdef DEBUG
    std::cout << gSimulationDataScintillatorArray->size() << " " << simulationDataScintillator << std::endl;
#endif
    //  }
  
  return true;
}

void ScintillatorSD::EndOfEvent(G4HCofThisEvent* /*HCTE*/)
{;}

